% Function to determine kinetic data using an inverse dynamics approach
% Written by Simon van der Helm, April 2019

function [M_ank,M_knee,M_hip,F_knee,F_hip,phi_knee,phi_ank] = CalcKinetics(isubject, i_trialfile, marker_trajectories, force_i_trialfile, FH_all, F_GRF, side)

global gender_subjects m_body_subjects;

if (lower(side) == 'l')
[~, FM2index] = intersect(marker_trajectories(i_trialfile).Labels, 'L-FM2');
else
[~, FM2index] = intersect(marker_trajectories(i_trialfile).Labels, 'R-FM2');
end
r_meta2 = squeeze(marker_trajectories(i_trialfile).Data(FM2index,1:3,:))'/1000;

r_knee = CalcKneeJointCentre(i_trialfile, marker_trajectories, side);
r_ank = CalcAnkleJointCentre(i_trialfile, marker_trajectories, side);
r_COP = CalculateCentreOfPressure(force_i_trialfile, side);

m_body = m_body_subjects(eval(isubject));
Ts=.01;%100 Hz sampling rate
gender = gender_subjects(eval(isubject));

[M_ank, M_knee, M_hip, F_knee, F_hip, phi_knee, phi_ank] = ...
    inverse_sagittal_dynamics(r_knee(:,[1,3]),...
    FH_all(:,[1,3])/1000, r_ank(:,[1,3]), r_meta2(:,[1,3]),...
    r_COP(:,[1,3]), F_GRF(:,[1,3]), m_body, gender, Ts);

function r_knee = CalcKneeJointCentre(i_trialfile, marker_trajectories, side)

if (lower(side) == 'l')
    [~, LFLEindex]=intersect(marker_trajectories(i_trialfile).Labels,'L-FLE');%% Lateral Epicondyle
    [~, LFMEindex]=intersect(marker_trajectories(i_trialfile).Labels,'L-FME');%% % Medial Epicondyle
    r_knee = squeeze(mean(marker_trajectories(i_trialfile).Data([LFLEindex, LFMEindex],:,:),1))'/1000;
else
    [~, RFLEindex] = intersect(marker_trajectories(i_trialfile).Labels,'R-FLE');%% Lateral Epicondyle
    [~, RFMEindex] = intersect(marker_trajectories(i_trialfile).Labels,'R-FME');%% % Medial Epicondyle
    r_knee = squeeze(mean(marker_trajectories(i_trialfile).Data([RFLEindex, RFMEindex],:,:),1))'/1000;
end

function [M_ank,M_knee,M_hip,F_knee,F_hip,phi_knee,phi_ank] = inverse_sagittal_dynamics(r_knee,r_hip,r_ank,r_meta2,r_COP,F_GRF,m_body,gender,Ts)
%assuming all inputs have nsample rows and 2 columns (X and Z coordinates)
g=9.81*0;%m/s^2
numsamples=size(r_knee,1);

%segment lengths, needed for inertias (scalars):
l_ul = nanmedian(sum((r_hip-r_knee).^2,2));%segment length upper leg (thigh)
l_sh = nanmedian(sum((r_knee-r_ank).^2,2));%segment length shank

male=(gender=='m');
%de Leva parameters:
if male==1
    l_ft=258.1/1000;%m, length of foot from de Leva, as we had no toe marker
    
    d_hip2Cul_s=.4095;% scaled distance from hip joint center to CoM thigh
    d_knee2Csh_s=.4352; %.. knee joint center fo CoM shank
    %we don't have the tip of the foot, can estimate from metatarsal
    d_ankle2Cf_s = .4415; %this is actually heel to CoM, but assuming ratio is similar as distance ankle joint to metatarsal
    
    %radius of gyration for each segment:
    r_ul_s=.329;
    r_sh_s=.251;
    r_ft_s=.257;
    
    %masses, scaled with body weight:
    m_ul_s=14.16/100;
    m_sh_s=4.33/100;
    m_ft_s=1.37/100;
    
else%female
    l_ft=228.3/1000;%m, length of foot from de Leva, as we had no toe marker
    
    d_hip2Cul_s=.3612;% scaled distance from hip joint center to CoM thigh
    d_knee2Csh_s=.4395; %.. knee joint center fo CoM shank
    %we don't have the tip of the foot, can estimate from metatarsal
    d_ankle2Cf_s = .4014; %this is actually heel to CoM, but assuming ratio is similar as distance ankle joint to metatarsal
    
    %radius of gyration for each segment:
    r_ul_s=.369;
    r_sh_s=.267;
    r_ft_s=.299;
    
    %masses, scaled with body weight:
    m_ul_s=14.78/100;
    m_sh_s=4.81/100;
    m_ft_s=1.29/100;    
end

%radius of gyration for each segment:
r_ul=r_ul_s*l_ul;
r_sh=r_sh_s*l_sh;
r_ft=r_ft_s*l_ft;

%masses of each segment:
m_ul=m_ul_s*m_body;
m_sh=m_sh_s*m_body;
m_ft=m_ft_s*m_body;

%CoM locations:
r_C_ul=r_hip+d_knee2Csh_s.*(r_knee-r_hip); %position vector CoM upper leg 
r_C_sh=r_knee+d_hip2Cul_s.*(r_ank-r_knee); %position vector CoM shank
r_C_ft=r_ank+d_ankle2Cf_s.*(r_meta2-r_ank); %position vector CoM shank, in direction of metatarsal

%inertias:
I_C_ul=r_ul.^2.*m_ul;%should be both scalars, so just in case
I_C_sh=r_sh.^2.*m_sh;
I_C_ft=r_ft.^2.*m_ft;


%using inertial coordinate system: X pointing forward, Z pointing up (so Y
%left)

%sum of moments about a segment's center of mass equals rate of change of
%centroidal angular momentum:
%sum M_C=dot(H)_C
%in the special case where the movement is planar, this reduces to
%sum M_C=I_C*dot(omega)
%Newton's Second Law:
%sum F= m_a_C
%abbreviations: sh: lowerleg (foot and shank), ul: thigh

%prepare moment arms:
r_Cft2ank=r_ank-r_C_ft;%vector COM foot to ankle joint
r_Cft2COP=r_COP-r_C_ft;%vector COM foot to CoP
r_Csh2knee=r_knee-r_C_sh;%vector from CoM shank to knee
r_Csh2ank=r_ank-r_C_sh;%vector from CoM shank to ankle joint
r_Cul2hip=r_hip-r_C_ul;%vector from CoM upper leg to hip
r_Cul2knee=r_knee-r_C_ul;%vector from CoM upper leg to knee

%angles with respect to pos Z-axis (so segment pointing up is angle=0):
phi_ul=atan2(r_hip(:,1)-r_knee(:,1),r_hip(:,2)-r_knee(:,2));
phi_sh=atan2(r_knee(:,1)-r_ank(:,1),r_knee(:,2)-r_ank(:,2));
%foot angle with respect to pos X-axis (so neutral pos is angle=0):
phi_ft=atan2(r_ank(:,1)-r_meta2(:,1),r_ank(:,2)-r_meta2(:,2))+pi/2;

%angular velocities (gradient can deal with NaN):
[omega_ul]=gradient(phi_ul,Ts);
[omega_sh]=gradient(phi_sh,Ts);
[omega_ft]=gradient(phi_ft,Ts);

%angular accelerations:
[domega_ul]=gradient(omega_ul*0,Ts);
[domega_sh]=gradient(omega_sh*0,Ts);
[domega_ft]=gradient(omega_ft*0,Ts);

%velocities of CoMs:
%CoM locations:
[~,v_C_ul]=gradient(r_C_ul,Ts);
[~,v_C_sh]=gradient(r_C_sh,Ts);
[~,v_C_ft]=gradient(r_C_ft,Ts);

%accelerations:
[~,a_C_ul]=gradient(v_C_ul*0,Ts);
[~,a_C_sh]=gradient(v_C_sh*0,Ts);
[~,a_C_ft]=gradient(v_C_ft*0,Ts);

% %Derivations in 3D, so with XYZ components (but assuming planar dynamics):
% %F_knee defined as vector acting on the shank, so -F_knee acts on the
% %upper leg, same for M_knee the torque vector acting on the shank:
% %so knee forces and torques follow from:
% F_knee + m_sh*g*[0;0;-1] + F_GRF = m_sh*a_C_sh;%Newton's Second Law
% M_knee + cross(r_Csh2knee,F_knee)+cross(r_C_sh2COP,F_GRF)...
%                             =I_C_sh*domega_sh     %Euler's Second Law
%
% %hip forces and torques follow from:
% F_hip + m_ul*g*[0;0;-1] - F_knee = m_ul*a_C_ul
% M_hip - M_knee + cross(r_C_ul2hip,F_hip)+cross(r_C_ul2knee,-F_knee)...
%                             =I_C_ul*domega_ul
%
% %%
% %solving the equations for the knee:
% F_knee = - m_sh*g*[0;0;-1] - F_GRF + m_sh*a_C_sh;
% M_knee= -cross(r_Csh2knee,F_knee)-cross(r_C_sh2COP,F_GRF)...
%                             +I_C_sh*domega_sh
% %and for the hip:
% F_hip = - m_ul*g*[0;0;-1] + F_knee + m_ul*a_C_ul
% M_hip = M_knee - cross(r_C_ul2hip,F_hip) - cross(r_C_ul2knee,-F_knee)...
%                              +I_C_ul*domega_ul

%%
%rewrite to process matrices with number of rows= number of samples and
%just containing sagittal-plane entries as columns, i.e.
%r_C_sh=[r_C_sh_X, r_C_sh_Z]
%a_C_sh=[a_C_sh_X, r_C_sh_Z]
%F_GRF=[F_GRF_X,F_GRF_Z]
%F_knee=[F_knee_X,F_knee_Z]
%M_knee=M_knee_Y (so a positive knee moment causes the knee to flex,
%and a positive hip moment causes the hip to extend)

%matrix form equations for the foot (F_ft acts on foot, -Fft on shank):
F_ank = - repmat(m_ft*g*[0,-1],numsamples,1) - F_GRF + m_ft*a_C_ft;
M_ank= - cross2D(r_Cft2ank,F_ank)  -cross2D(r_Cft2COP,F_GRF)...
    +I_C_ft*domega_ft;

%matrix form equations for the knee:
F_knee = - repmat(m_sh*g*[0,-1],numsamples,1) + F_ank + m_sh*a_C_sh;
M_knee= M_ank- cross2D(r_Csh2knee,F_knee)  -cross2D(r_Csh2ank,-F_ank)...
    +I_C_sh*domega_sh;

%and for the hip:
F_hip = - repmat(m_ul*g*[0,-1],numsamples,1) + F_knee + m_ul*a_C_ul;
M_hip = M_knee - cross2D(r_Cul2hip,F_hip) - cross2D(r_Cul2knee,-F_knee)...
    +I_C_ul*domega_ul;

%reconstruct also knee angle:
phi_knee=phi_sh-phi_ul;
phi_ank=phi_ft-phi_sh;

function [MY]= cross2D(r,F)
%2D cross product if the X and Z components of the vectors are given:
MY=r(:,2).*F(:,1)-r(:,1).*F(:,2);

function r_ank = CalcAnkleJointCentre(i_trialfile, marker_trajectories, side)

if (lower(side) == 'l')
    [~, LTAMindex]=intersect(marker_trajectories(i_trialfile).Labels,'L-TAM');% Medial Malleolus
    [~, LFALindex]=intersect(marker_trajectories(i_trialfile).Labels,'L-FAL');%% % Lateral Malleolus
    r_ank= squeeze(mean(marker_trajectories(i_trialfile).Data([LTAMindex,LFALindex],:,:),1))'/1000;
else
    [~ ,RTAMindex]=intersect(marker_trajectories(i_trialfile).Labels,'R-TAM');% Medial Malleolus
    [~, RFALindex]=intersect(marker_trajectories(i_trialfile).Labels,'R-FAL');%% % Lateral Malleolus
    r_ank = squeeze(mean(marker_trajectories(i_trialfile).Data([RTAMindex,RFALindex],:,:),1))'/1000;
end

function r_COP = CalculateCentreOfPressure(force_i_trialfile, side)

if (lower(side) == 'l')
    r_COP = force_i_trialfile(2).COP'/1000;
else
    r_COP = force_i_trialfile(3).COP'/1000;
end