%% Function to build a local coordinate system for a body segment
% Written by Simon van der Helm, April 2019
%
% The provided data in the function should be either 29 markers long,
% or the specific markers required for the analysis. The order of the
% markers should be:
% All:
% 'LA' 'RA' 'C7' 'IJ' 'L-IPS' 'R-IPS' 'L-IAS' 'R-IAS' 'L-FTC' 'R-FTC'
% 'L-FLE' 'L-FAX' 'L-TTC' 'L-FME' 'R-FME' 'R-FLE' 'R-TTC' 'L-FCC' 'R-FCC'
% 'L-FAL' 'L-TAM' 'R-TAM' 'R-FAL' 'R-FM5' 'L-FM5' 'L-FM2' 'L-FM1' 'R-FM1'
% 'R-FM2'
% Pelvis:
% 'L-IAS' 'R-IAS' 'L-IPS' 'R-IPS'
% Thigh (Left or Right):
% '~-FLE' '~-FME' 'L-IAS' 'R-IAS' 'L-IPS' 'R-IPS'
% Shank (Left or Right):
% '~-TAM' '~-FAL' '~-FAX' '~-TTC'
% Foot (Left or Right):
% '~-FCC' '~-FM1' '~-FM2' '~-FM5'
% The provided data could also be provided as a struct with a data field
% and a labels field.
%
% The segment should be from the set:
% All - All available local coordinate systems will be returned in a struct
% Pelvis - The local coordinate system of the Pelvis will be returned by
% an origin and a Rotation Matrix
% LThigh - The local coordinate system of the Left Thigh will be returned by
% an origin and a Rotation Matrix
% RThigh - The local coordinate system of the Right Thigh will be returned by
% an origin and a Rotation Matrix
% LShank - The local coordinate system of the Left Shank will be returned by
% an origin and a Rotation Matrix
% RShank - The local coordinate system of the Right Shank will be returned by
% an origin and a Rotation Matrix
% LFoot - The local coordinate system of the Left Foot will be returned by
% an origin and a Rotation Matrix
% RFoot - The local coordinate system of the Right Foot will be returned by
% an origin and a Rotation Matrix

function [coordSys] = MakeLocalCoord(data,segment,varargin)

% Check which side is the prosthetic side
if isempty(varargin)
    if ismember('L-FAX',data.Labels)
        pSide = 'R';
    else
        pSide = 'L';
    end
else
    pSide = varargin{1};
end

if isstruct(data) 
    labels = data.Labels; % If the data is provided as a structure, extract the labels and the data
    data = data.Data;
elseif size(data,1)==29
    % If the data is provided as a matrix, use the predetermined labels
    labels = {'LA' 'RA' 'C7' 'IJ' 'L-IPS' 'R-IPS' 'L-IAS' 'R-IAS' 'L-FTC' 'R-FTC'
              'L-FLE' 'L-FAX' 'L-TTC' 'L-FME' 'R-FME' 'R-FLE' 'R-TTC' 'L-FCC' 'R-FCC'
              'L-FAL' 'L-TAM' 'R-TAM' 'R-FAL' 'R-FM5' 'L-FM5' 'L-FM2' 'L-FM1' 'R-FM1'
              'R-FM2'};
else
    switch segment
        case 'Pelvis'
            labels = {'L-IAS' 'R-IAS' 'L-IPS' 'R-IPS'};
        case 'LThigh'
            labels = {'L-FLE' 'L-FME' 'L-IAS' 'R-IAS' 'L-IPS' 'R-IPS'};
        case 'RThigh'
            labels = {'R-FLE' 'R-FME' 'L-IAS' 'R-IAS' 'L-IPS' 'R-IPS'};
        case 'LShank'
            labels = {'L-TAM' 'L-FAL' 'L-FAX' 'L-TTC'};
        case 'RShank'
            labels = {'R-TAM' 'R-FAL' 'R-FAX' 'R-TTC'};
        case 'LFoot'
            labels = {'L-FCC' 'L-FM1' 'L-FM2' 'L-FM5'}; 
        case 'RFoot'
            labels = {'R-FCC' 'R-FM1' 'R-FM2' 'R-FM5'};
    end
end

coordSys = struct([]);

% Check if the Pelvis coordinate system should be built
if ismember(segment,{'Pelvis','All'})
    % Select the markers
    [~,IData,~] = intersect(labels,{'L-IAS','R-IAS','L-IPS','R-IPS'});
    % Find the first instance that all pelvis markers are visible
    coordSys(1).t_Pelvis = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
    
    if ~isempty(coordSys(1).t_Pelvis)
        % Make the pelvis coordinate system
        [coordSys(1).O_Pelvis,coordSys(1).R_Pelvis] = makePelvisCoord(data(:,:,coordSys(1).t_Pelvis),labels);
    end
end

% Check if the left thigh coordinate system should be built
if ismember(segment,{'LThigh','All'})
    if isfield(coordSys,'R_Pelvis') % If the pelvis coordinate system exists
        % Select the markers
        [~,~,IData] = intersect({'L-IAS','L-IPS','R-IAS','R-IPS','L-FLE','L-FME','L-FTC'},labels,'stable');
        % Find the first instance enough left thigh markers are visible
        if numel(IData)<7%this is probably subject 5, where L_FTC is missing (and also R-IAS is missing but that one gets reconstructed)
            coordSys(1).t_LThigh = find(and(~any(any(isnan(data(IData(5:6),1:3,:)),2),1),sum(any(isnan(data(IData(1:4),1:3,:)),2),1)<=1),1);
        else
            coordSys(1).t_LThigh = find(and(~any(any(isnan(data(IData(5:7),1:3,:)),2),1),sum(any(isnan(data(IData(1:4),1:3,:)),2),1)<=1),1);
        end
        % Find the local coordinates of the markers in the pelvic
        % coordinate system
        X = localCoord(struct('Data',{data},'Labels',{labels}),coordSys);
        % Construct the coordinate systems (primarily pelvic) based on the
        % local coordinates
        temp = findLocalCoord(struct('Data',{data(:,:,coordSys(1).t_LThigh)},'Labels',{labels}),X);
    else
        % Select the markers
        [~,~,IData] = intersect({'L-IAS','L-IPS','R-IAS','R-IPS','L-FLE','L-FME','L-FTC'},labels);
        % Find the first instance all left thigh markers are visible
        coordSys(1).t_LThigh = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
        temp.t = 'Haha'; % ?
    end
    if ~isempty(coordSys(1).t_LThigh)
        % Construct the local thigh coordinate system
        [coordSys(1).O_LThigh,coordSys(1).R_LThigh] = makeThighCoord(data(:,:,coordSys.t_LThigh),labels,'L-',temp,X);
    end
end

% Check if the right thigh coordinate system should be built
if ismember(segment,{'RThigh','All'})
    if isfield(coordSys,'R_Pelvis') % If the pelvic coordinate system exists
        % Find the right thigh markers
        [~,~,IData] = intersect({'L-IAS','L-IPS','R-IAS','R-IPS','R-FLE','R-FME','R-FTC'},labels,'stable'); 
        % Find the first instance enough right thigh markers are visible
        coordSys(1).t_RThigh = find(and(~any(any(isnan(data(IData(5:7),1:3,:)),2),1),sum(any(isnan(data(IData(1:4),1:3,:)),2),1)<=1),1);
        % Find the local marker coordinates in the pelvic coordinate system
        X = localCoord(struct('Data',{data},'Labels',{labels}),coordSys);
        % Find the current pelvic coordinate system
        temp = findLocalCoord(struct('Data',{data(:,:,coordSys(1).t_RThigh)},'Labels',{labels}),X);
    else
        % Find the right thigh markers
        [~,~,IData] = intersect({'L-IAS','L-IPS','R-IAS','R-IPS','R-FLE','R-FME','R-FTC'},labels);
        % Find the first instance all right thigh markers are visible
        coordSys(1).t_RThigh = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
        temp.t = 'Haha'; % ?
    end
    if ~isempty(coordSys(1).t_RThigh)
        % Construct the right thigh coordinate system
        [coordSys(1).O_RThigh,coordSys(1).R_RThigh] = makeThighCoord(data(:,:,coordSys.t_RThigh),labels,'R-',temp,X);
    end
end

% Check if the left shank coordinate system should be built
if ismember(segment,{'LShank','All'})
    if strcmp(pSide,'L')
        % Find the prosthetic side shank markers
        [~,IData,~] = intersect(labels,{'L-TAM','L-FAL','L-FME','L-FLE'});
        % Find the first instance all markers are visible
        coordSys(1).t_LShank = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
        if ~isempty(coordSys(1).t_LShank)
            % Construct the coordinate system
            [coordSys(1).O_LShank,coordSys(1).R_LShank] = makeProstheticCoord(data(:,:,coordSys(1).t_LShank),labels,'L-');
        end
    else
        % Find the healthy side shank markers
        if ismember('L-TTC',labels)
            % FAX and TTC are better markers for the shank coordinate
            % system
            [~,IData,~] = intersect(labels,{'L-TAM','L-FAL','L-FAX','L-TTC'});
        else
            % FME and FLE suffice if FAX and TTC are not available
            [~,IData,~] = intersect(labels,{'L-TAM','L-FAL','L-FME','L-FLE'});
        end
        % Find the first instance all markers are visible
        coordSys(1).t_LShank = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
        if ~isempty(coordSys(1).t_LShank)
            % Construct the coordinate system
            [coordSys(1).O_LShank,coordSys(1).R_LShank] = makeShankCoord(data(:,:,coordSys(1).t_LShank),labels,'L-');
        end
    end
end

% Check if the right shank coordinate system should be built
if ismember(segment,{'RShank','All'})
    if strcmp(pSide,'R')
        % Find the prosthetic shank markers
        [~,IData,~] = intersect(labels,{'R-TAM','R-FAL','R-FME','R-FLE'});
        % Find the first instance all markers are visible
        coordSys(1).t_RShank = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
        if ~isempty(coordSys(1).t_RShank)
            % Construct the local coordinate system
            [coordSys(1).O_RShank,coordSys(1).R_RShank] = makeProstheticCoord(data(:,:,coordSys(1).t_RShank),labels,'R-');
        end
    else
        if ismember('R-TTC',labels)
            % TTC and FAX are better for the shank coordinate system
            [~,IData,~] = intersect(labels,{'R-TAM','R-FAL','R-FAX','R-TTC'});
        else
            % FME and FLE suffice for the shank coordinate system
            [~,IData,~] = intersect(labels,{'R-TAM','R-FAL','R-FME','R-FLE'});
        end
        % Find the first instance all markers are visible
        coordSys(1).t_RShank = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
        if ~isempty(coordSys(1).t_RShank)
            % Construct the local coordinate system
            [coordSys(1).O_RShank,coordSys(1).R_RShank] = makeShankCoord(data(:,:,coordSys(1).t_RShank),labels,'R-');
        end
    end
end

% Check if the left foot coordinate system should be built
if ismember(segment,{'LFoot','All'})
    % Find the left foot markers
    [~,IData,~] = intersect(labels,{'L-FCC','L-FM1','L-FM2','L-FM5'});
    % Find the first instance all markers are visible
    coordSys(1).t_LFoot = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
    if ~isempty(coordSys(1).t_LFoot)
        % Construct the coordinate system
        [coordSys(1).O_LFoot,coordSys(1).R_LFoot] = makeFootCoord(data(:,:,coordSys(1).t_LFoot),labels,'L-');
    end
end

% Check if the right foot coordinate system should be built
if ismember(segment,{'RFoot','All'})
    % Find the right foot markers
    [~,IData,~] = intersect(labels,{'R-FCC','R-FM1','R-FM2','R-FM5'});
    % Find the first instance all markers are visible
    coordSys(1).t_RFoot = find(~any(any(isnan(data(IData,1:3,:)),2),1),1);
    if ~isempty(coordSys(1).t_RFoot)
        % Construct the local coordinate system
        [coordSys(1).O_RFoot,coordSys(1).R_RFoot] = makeFootCoord(data(:,:,coordSys(1).t_RFoot),labels,'R-');
    end
end

%% Function to make a coordinate system for the Thigh Segment
% Written by Simon van der Helm

function [O,R] = makeThighCoord(data,labels,direction,varargin)

if ~ismember([direction,'FH'],labels) % Check if the femoral head is present in the data
    if length(varargin) == 2 % Check if the pelvis coordinate system is provided
        R_Pelvis = varargin{1}.R_Pelvis; % Pelvis coordinate system is in the first optional argument
        O_Pelvis = varargin{1}.O_Pelvis;
        X = varargin{2}; % Local coordinates of the markers in the pelvic coordinate system
        lFH = X.Pelvis.Data(strcmp(X.Pelvis.Labels,[direction,'FH']),1:3); % Find the local coordinates of the femoral head
        FH = O_Pelvis+R_Pelvis*lFH'; % Calculate the global coordinates of the femoral head
    elseif ~isempty(varargin) && isfield(varargin{1},'R_Pelvis')
%       R_Pelvis = varargin{1}.R_Pelvis;
        FH = findFemoralHead(direction(1),struct('Data',{data},'Labels',{labels}),varargin{1}); % Calculate the global coordinates of the femoral head
    else
        FH = findFemoralHead(direction(1),struct('Data',{data},'Labels',{labels})); % Calculate the global coordinates of the femoral head
    end
else
    FH = data(strcmp(labels,[direction,'FH']),1:3)';
end

LE = data(strcmp(labels,[direction,'FLE']),1:3)'; % Lateral Epicondyle
ME = data(strcmp(labels,[direction,'FME']),1:3)'; % Medial Epicondyle
O = LE+(ME-LE)/2; % Origin betweeen epicondyles
x = FH-O; % x-axis pointing in the direction of the femoral head
x = x/norm(x); % Normalize the x-axis

% This step is different for the left and right thigh
switch direction
    case 'L-'
        tempz = (LE-O)/norm(LE-O); % z-axis always pointing right when viewed from the front
    case 'R-'
        tempz = (ME-O)/norm(ME-O);
end
        
y = cross(tempz,x); % y-axis perpendicular to x and temporary z
y = y/norm(y);
z = cross(x,y); % z-axis perpendicular to x and y
z = z/norm(z);

R = [x,y,z]; % Rotation Matrix

%% Function to make a coordinate system for the Shank Segment
% Written by Simon van der Helm

function [O,R] = makeShankCoord(data,labels,direction)

MM = data(strcmp(labels,[direction,'TAM']),1:3)'; % Medial Malleolus
LM = data(strcmp(labels,[direction,'FAL']),1:3)'; % Lateral Malleolus
HF = data(strcmp(labels,[direction,'FAX']),1:3)'; % Head of Fibula
TT = data(strcmp(labels,[direction,'TTC']),1:3)'; % Tibial Tuberosity
% if size([MM LM HF TT],2) < 4
%     O = NaN(3,1);
%     R = NaN(3);
%     return
% end
if isempty(TT) % Check if the Tibial Tuberosity exists
    % Use Epicondyles in stead
    ME = data(strcmp(labels,[direction,'FME']),1:3)';
    LE = data(strcmp(labels,[direction,'FLE']),1:3)';
    TT = ME+(LE-ME)/2;
end
    
O = MM + (LM-MM)/2; % Origin

if strcmp(direction,'L-')
    tempz = (LM-O)/norm(LM-O); % Temporary z-axis
else
    tempz = (MM-O)/norm(MM-O);
end

tempx = (HF-O)/norm(HF-O); % Temporary x-axis
QF = cross(tempz,tempx); % QF vector, perpendicular to z and x-axis
temp = (TT-O)/norm(TT-O); % Temporary vector from origin to tibial tuberosity
QS = cross(temp,QF); % QS vector, perpendicular to temp and QF
x = cross(QF,QS); % x-axis perpendicular to QF and QS
x = x/norm(x);
z = cross(x,QF); % z-axis perpendicular to x and QF
z = z/norm(z);
y = cross(z,x); % y-axis perpendicular to z and x
y = y/norm(y);

R = [x,y,z]; % Construct the rotation matrix

%% Function to make a coordinate system for the Thigh Segment
% Written by Simon van der Helm

function [O,R] = makeProstheticCoord(data,labels,direction)

MM = data(strcmp(labels,[direction,'TAM']),1:3)'; % Medial Malleolus
LM = data(strcmp(labels,[direction,'FAL']),1:3)'; % Lateral Malleolus
ME = data(strcmp(labels,[direction,'FME']),1:3)'; % Medial Epicondyle
LE = data(strcmp(labels,[direction,'FLE']),1:3)'; % Lateral Epicondyle
if size([MM LM ME LE],2) < 4 % Check for enough points
    O = NaN(3,1);
    R = NaN(3);
    return
end
    
O = MM + (LM-MM)/2; % Origin between malleoli
M = LE + (ME-LE)/2; % M between epicondyles

if strcmp(direction,'L-') % z should point right when viewed from the front
    tempz = (LM-O)/norm(LM-O); 
else
    tempz = (MM-O)/norm(MM-O);
end

x = (M-O); % x-axis
x = x/norm(x);
y = cross(tempz,x); % y-axis perpendicular to x and temporary z
y = y/norm(y);
z = cross(x,y); % z-axis perpendicular to x and y
z = z/norm(z);

R = [x,y,z]; % Rotation matrix

%% Function to make a coordinate system for the Thigh Segment
% Written by Simon van der Helm

function [O,R] = makeFootCoord(data,labels,direction)

CA = data(strcmp(labels,[direction,'FCC']),1:3)'; % Calcaneus
FM1 = data(strcmp(labels,[direction,'FM1']),1:3)'; % First Metatarsal
FM2 = data(strcmp(labels,[direction,'FM2']),1:3)'; % Second Metatarsal
FM5 = data(strcmp(labels,[direction,'FM5']),1:3)'; % Fifth Metatarsal
O = CA; % Origin in calcaneus
temp1 = (FM1-O); % Temporary vectors to first and fifth metatarsal
temp5 = (FM5-O);

%HV: problem: in subjects 4 and 5, the first and fifth metatarsal are
%exchanged. 

if strcmp(direction,'L-') % Direction should point up
        QT = cross(temp1,temp5);
else
        QT = cross(temp5,temp1);
end

temp2 = FM2-O; % Temporary vector from origin to second metatarsal
%HV: mistake corrected: 
% The comment said "Temporary vector from second metatarsal to origin"
% but the variable was  FM2-O. So there was an 180 deg mismatch. 
% corrected the comment

QS = cross(QT,temp2); % Temporary vector, perpendicular to QT and second metatarsal vector
y = cross(QS,QT); % y-axis perpendicular to Qs and QT
y = y/norm(y);
z = cross(QT,y); % z-axis perpendicular to QT and y
z = z/norm(z);
x = cross(y,z); % x-axis perpendicular to y and z
x = x/norm(x);

R = [x,y,z]; % Construct Rotation Matrix
