%% Function to find the Femoral Head, based on the Markers on the Pelvis
% Written by Simon van der Helm, April 2019
%
% [FH] = findFemoralHead(side,data)
% FH is the 3x1 position of the femoral head, in the global coordinate
% system. side is either 'L' or 'R', to define the side of the Femoral
% Head. data is a structure containing the 4 pelvis markers
% (L-IAS,R-IAS,L-IPS,R-IPS) and corresponding labels.
%
% [FH] = findFemoralHead(side,data,R_Pelvis)
% FH is the 3x1 position of the femoral head, in the global coordinate
% system. side is either 'L' or 'R', to define the side of the Femoral
% Head. data is a structure containing at least 2 pelvis markers (L-IAS and
% R-IAS) and corresponding labels. R_Pelvis is the rotation matrix, mapping
% from the local Pelvis coordinate system, to the global coordinate system

function [FH] = findFemoralHead(side,data,varargin)

if isempty(varargin)
    [O_Pelvis,R_Pelvis] = makePelvisCoord(data.Data,data.Labels); % Build the pelvis coordinate system
    bLocal = 'global'; % Global coordinates requested (unless specified otherwise)
elseif length(varargin)==1
    O_Pelvis = varargin{1}.O_Pelvis; % Pelvis coordinate system provided
    R_Pelvis = varargin{1}.R_Pelvis;
    bLocal = 'global';
else
    if isfield(varargin{1},'R_Pelvis') % Pelvis coordinate system provided
        R_Pelvis = varargin{1}.R_Pelvis;
        O_Pelvis = varargin{1}.O_Pelvis;
    elseif isfield(varargin{1},['R_',side,'Thigh']) % Thigh coordinate system provided
        eval(['R = varargin{1}.R_',side,'Thigh;']);
        eval(['O = varargin{1}.O_',side,'Thigh;']);
    else
        FH = [NaN;NaN;NaN]; % Wrong optional arguments
        return
    end
    bLocal = varargin{2}; % Second argument defines whether local position is wanted
end

if strcmp(bLocal,'global')
    LASIS = data.Data(strcmp(data.Labels,'L-IAS'),1:3)'; % Left Anterior Superior Iliac Spine
    RASIS = data.Data(strcmp(data.Labels,'R-IAS'),1:3)'; % Right Anterior Superior Iliac Spine
    dASIS = norm(LASIS-RASIS); % Distance

    if side == 'L' % What side femoral head to we want?
        FH = LASIS-R_Pelvis*[0.304*dASIS;0.193*dASIS;0.141*dASIS]; % Calculate femoral head position based on average anatomical position
    else
        FH = RASIS-R_Pelvis*[0.304*dASIS;0.193*dASIS;-0.141*dASIS]; % Calculate femoral head position based on average anatomical position
    end
    %numbers taken from table 4 in: Bell, A. L., Pedersen, D. R., & Brand, R. A. (1990).
    %A comparison of the accuracy of several hip center location prediction methods.
    %Journal of Biomechanics, 23(6), 617621. doi:10.1016/0021-9290(90)90054-7 
else
    if isfield(varargin{1},'R_Pelvis') % If pelvis coordinate system was provided
        lFH = data.Pelvis.Data(strcmp(data.Pelvis.Labels,[side,'-FH']),1:3); % Global position of the femoral head
        FH = O_Pelvis+R_Pelvis*lFH'; % Calculate local position of the Femoral head
    elseif isfield(varargin{1},['R_',side,'Thigh']) % If thigh coordinate system was provided
        marker = [side,'-FH']; % Find femoral head 'marker'
        lFH = data.([side,'Thigh']).Data(strcmp(data.([side,'Thigh']).Labels,marker),1:3); % Extract global position
        FH = O+R*lFH'; % Calculate local position of the Femoral Head
    end
end