%% Function to find a local coordinate system based on points on the segment
% Written by Simon van der Helm, April 2019
%
% [coordSys] = findLocalCoord(data,X)
% coordSys is a structure containing a Rotation Matrix R_<Name_Segment> and
% an Origin Vector O_<Name_Segment>. data is a structure containing a field
% called Data with marker positions and a field called Labels, with the
% corresponding marker names.

function [coordSys] = findLocalCoord(data,X)
sorted = 'stable';
segments = fieldnames(X); % Check which segments are included in the local coordinates structure
coordSys = struct();
for i = 1:length(segments)
    [markers, IX, Idata] = intersect(X.(segments{i}).Labels, data.Labels, sorted); % Find the markers on the segment
    gPos = data.Data(Idata,1:3)'; % Find the global locations of the markers
    gNaN = find(any(isnan(gPos))); 
    eval(['lPos = X.',segments{i},'.Data(IX,1:3);']); % Find the local coordinates of the markers
    lPos = lPos';
    lNaN = find(any(isnan(lPos)));
    gPos(:,union(gNaN,lNaN)) = []; % Remove instances that have NaN's (global or local)
    lPos(:,union(gNaN,lNaN)) = [];
    markers(union(gNaN,lNaN)) = [];
    % Find all markers that are not medial or lateral epicondyles
    [nMarkers,Im] = setdiff(markers,{'L-FLE','L-FME','R-FME','R-FLE'},sorted);
    nGPos = gPos(:,Im); % Global positions of markers
    nLPos = lPos(:,Im); % Local positions of markers
    if length(nMarkers)>= 3 % Check if enough markers
        % Calculate rotation matrix and origin for all 
        for j = 2:length(nMarkers)
            w(:,j-1) = nGPos(:,j)-nGPos(:,1);  % Vector of global positions compared to the first marker
            v(:,j-1) = nLPos(:,j)-nLPos(:,1);  % Vector of local positions compared to the first marker
        end
        B = w*v'; 
        [U,~,V] = svd(B); % Singular value decomposition
        M = diag([1,1,det(U)*det(V)]);
        R = U*M*V'; % Calculate rotation matrix
        O = nGPos-R*nLPos; % Find all 'Origins'
        O = mean(O,2); % Average the origins
        coordSys.(['R_',segments{i}])=R;
        coordSys.(['O_',segments{i}])=O;
    elseif length(markers) >= 3 % If enough with the femoral epicondyles
        for j = 2:length(markers)
            w(:,j-1) = gPos(:,j)-gPos(:,1); % Vector of global positions compared to first marker
            v(:,j-1) = lPos(:,j)-lPos(:,1); % Vector of local positions compared to first marker
        end
        B = w*v'; 
        [U, ~, V] = svd(B); % Singular value decomposition
        M = diag([1, 1, det(U)*det(V)]);
        R = U*M*V'; % Calculate rotation matrix
        O = gPos-R*lPos; % Find all 'Origins'
        O = mean(O,2); % Average the origins
        coordSys.(['R_',segments{i}])=R;
        coordSys.(['O_',segments{i}])=O;
    end
    % If we have evaluated the pelvis segment, add femoral head location
    % for thigh segment evaluation
    if strcmp(segments{i},'Pelvis') && isfield(coordSys,'R_Pelvis')
        % Find femoral head position
        data.Data(end+1,1:3) = findFemoralHead('L',X,coordSys,'local');
        data.Data(end+1,1:3) = findFemoralHead('R',X,coordSys,'local');
        
        % Save femoral head position
        data.Labels{end+1} = 'L-FH';
        data.Labels{end+1} = 'R-FH';
    end        
end   
    