%% Function to find the local coordinates of different marker positions
% Written by Simon van der Helm, April 2019
%
% [X] = localCoord(data,coordSys)
% X is a structure containing a structure for each segment. The segment
% structures contain a 1xN Labels cell array and a Nx3 Data array,
% with N being the number of markers on that section.
%
% data is a structure containing a 1xM Labels cell array and a Mx3 Data
% array, with M being the total number of markers.
%
% coordSys is a structure containing 3x1 origin vectors for each coordinate
% system (called O_<name_segment>) and 3x3 rotation matrices for each
% coordinate system (called R_<name_segment>).

function [X] = localCoord(data,coordSys)

segments = {'Pelvis';'LThigh';'RThigh';'LShank';'RShank';'LFoot';'RFoot'};
markers = {{'L-IAS','R-IAS','L-IPS','R-IPS'};{'L-FME','L-FLE','L-FTC','L-FH'};...
           {'R-FME','R-FLE','R-FTC','R-FH'};{'L-TTC','L-TAM','L-FAL','L-FAX','L-FME','L-FLE'};...
           {'R-TTC','R-TAM','R-FAL','R-FAX','R-FME','R-FLE'};{'L-FCC','L-FM1','L-FM2','L-FM5'};...
           {'L-FCC','L-FM1','L-FM2','L-FM5'}};
X = struct();       
       
for i = 1:length(segments)
    if isfield(coordSys,['R_',segments{i}]) % Check if the local coordinate system exists
        k = 1;
        for j = 1:length(markers{i})
            if ismember(markers{i}{j},data.Labels) % Check if the marker belongs to the segment
                X.(segments{i}).Data(k,:) = inv(coordSys.(['R_',segments{i}]))*(data.Data(strcmp(data.Labels,markers{i}{j}),1:3,coordSys.(['t_',segments{i}]))'-coordSys.(['O_',segments{i}])); % Construct the local coordinates
                % eval(['X.',segments{i},'.Labels{',num2str(k),'} = markers{i}{j};']); 
                X.(segments{i}).Labels{k} = markers{i}{j}; % Include the marker label
                k = k+1;
            end
        end
        switch i % Check which segment
            case 1 % Pelvis segment
                % Find femoral head position
                LFH = findFemoralHead('L',struct('Data',{data.Data(:,:,coordSys.t_Pelvis)},'Labels',{data.Labels}),coordSys); 
                RFH = findFemoralHead('R',struct('Data',{data.Data(:,:,coordSys.t_Pelvis)},'Labels',{data.Labels}),coordSys);
                % Save femoral head position
                X.Pelvis.Data(k,:) = inv(coordSys.R_Pelvis)*(LFH-coordSys.O_Pelvis);
                X.Pelvis.Data(k+1,:) = inv(coordSys.R_Pelvis)*(RFH-coordSys.O_Pelvis);
                X.Pelvis.Labels{k} = 'L-FH';
                X.Pelvis.Labels{k+1} = 'R-FH';
            case 2 % Left Thigh segment
                % Find femoral head position
                LFH = findFemoralHead('L',X,findLocalCoord(struct('Data',{data.Data(:,:,coordSys.t_LThigh)},'Labels',{data.Labels}),X),'local');
                % Save femoral head position
                X.LThigh.Data(k,:) = inv(coordSys.R_LThigh)*(LFH-coordSys.O_LThigh);
                X.LThigh.Labels{k} = 'L-FH';
            case 3 % Right Thigh segment
                % Find femoral head position
                RFH = findFemoralHead('R',X,findLocalCoord(struct('Data',{data.Data(:,:,coordSys.t_RThigh)},'Labels',{data.Labels}),X),'local');
                % Save femoral head position
                X.RThigh.Data(k,:) = inv(coordSys.R_RThigh)*(RFH-coordSys.O_RThigh);
                X.RThigh.Labels{k} = 'R-FH';                
        end
    end
end