%% M-file to process the data from user tests using the ERiK prosthesis
% Written by Simon van der Helm,
% Edited by H. Vallery in 2022
% Refactored and extended for work done values by Andrew P in 2023 for publication in Wearable Technologies

clear all; 
close all; 
clc;

%% global variables
global C joint_labels angle_labels m_body_subjects gender_subjects prosthesis_subjects;

C = 'brg';      % Colors
joint_labels = {'Left Hip','Right Hip','Left Knee','Right Knee','Left Ankle','Right Ankle'};
angle_labels = {'Flexion-Extension','Abduction-Adduction','Exorotation-Endorotation'};
m_body_subjects = [87, 80, 63, 61, 76, 60, 61];               % kg , mass of each subject
gender_subjects = ['f', 'm', 'm', 'f', 'f', 'm', 'f'];
prosthesis_subjects = ['r', 'l', 'l', 'r', 'l', 'l', 'l'];    % which side is the prosthetic side

%% script flags
recalc= 0 ;                     % re-process all the raw data, even if processed data exists
bAnimate = 0;                   % show the animation? (Takes longer)
plotallindividuals = 0;         % show plots for each person or just the aggregate data
plotKneeFlexionAndMoment = 0;

%% initialise vectors
weightratio_transit_vec=[];
boxplotlabel_vec=[];

%% find data from directory
% Find the contents of the Data folder
dataContents = dir('../Data_sel'); % Contents of the Data Folder
dataContents = Struct2Struct(dataContents); % Make single dimension structure
rangeFolders = find(strncmp(dataContents.name,'Subject',7))';%compare the first letters to reconize the right folders

%% initialise processedData cell
processedData = cell(length(rangeFolders), 2);

%% Process each folder
saveIndex = 0;
for ifolder = rangeFolders                          % Loop through folders and thereby subjects
    ifoldername = dataContents.name{ifolder};       % name of the subject folder
    isubject = ifoldername(9);                      % extract the subject's number from the folder name
    disp(['processing subject ', num2str(isubject)])
    folderContents = dir(['..\Data_sel\', dataContents.name{ifolder}]); % Find contents of the subject folder
    folderContents = Struct2Struct(folderContents); % Make single dimension structure
    if ~exist(['..\Data_sel\',dataContents.name{ifolder},'\processed'], 'dir')
        mkdir(['..\Data_sel\',dataContents.name{ifolder},'\processed']) % Create a folder for the trial if it did not exist
    end
    iStatic = find(and(startsWith(folderContents.name, 'static'), folderContents.isdir == 0)); % Check if a static test was done
    indices_matfiles = setdiff(find(endsWith(folderContents.name,'.mat')), iStatic); % Find other data files
    X = ExtractCoordinateSystem(isubject, folderContents, indices_matfiles);
    [marker_trajectories, force] = LoadTrialData(indices_matfiles, folderContents);

    for iTrialfile = indices_matfiles' % Loop through the trial data files
        trialname=folderContents.name(iTrialfile);
        trialname=trialname{1};

        marker_trajectories = ProcessMarkerTrajectories(isubject, iTrialfile, indices_matfiles, marker_trajectories);
        force = ProcessForce(isubject, iTrialfile, force);

        % Calculate the joint angles, ground reaction forces, trunk angle,
        % joint moments and trial start and ends
        force_i_trialfile = force(iTrialfile,:);
        marker_trajectories_i_trialfile = marker_trajectories(iTrialfile);
        [angles, theta, FA, FC, FD, MLA, MRA, MLK, MRK, MLH, MRH, cut, LFH_all, RFH_all] ...
            = makeJointAnglesandMoments([folderContents.folder{iTrialfile},'\', ...
            folderContents.name{iTrialfile}(1:end-4)], marker_trajectories_i_trialfile, force_i_trialfile, X, bAnimate);
        F_GRF_L = -force_i_trialfile(2).Force';
        F_GRF_R = -force_i_trialfile(3).Force';
        [M_ank_L, M_knee_L, M_hip_L, F_knee_L, F_hip_L, phi_knee_L, phi_ank_L] = CalcKinetics(isubject, iTrialfile, marker_trajectories, force_i_trialfile, LFH_all, F_GRF_L, 'l');
        [M_ank_R, M_knee_R, M_hip_R, F_knee_R, F_hip_R, phi_knee_R, phi_ank_R] = CalcKinetics(isubject, iTrialfile, marker_trajectories, force_i_trialfile, RFH_all, F_GRF_R, 'r');

        %classify the times:
        F_GRF_chair=-force_i_trialfile(1).Force';

        [knee_flexion_sound, knee_flexion_pros, M_knee_s, M_knee_p, M_hip_s, M_hip_p, F_GRF_sound, F_GRF_pros] = ...
            SelectProsAndSound(isubject, phi_knee_L, phi_knee_R, M_knee_L, M_knee_R, M_hip_L, M_hip_R, F_GRF_L, F_GRF_R);
        [indices_standing_locked_knee, indices_sitting, indices_lowweightonfeet, indices_seatcontact] = CalcIndices(F_GRF_chair, F_GRF_sound, F_GRF_pros, knee_flexion_pros);
        [knee_flexion_sound, knee_flexion_pros] = CalibrateKneeFlexion(indices_standing_locked_knee, knee_flexion_sound, knee_flexion_pros);

        % no reliable clculation of hip torque when touching the chair, so:
        M_hip_R(indices_seatcontact,:) = NaN;
        M_hip_L(indices_seatcontact,:) = NaN;

        indices_transit = CalcIndicesTransit(indices_standing_locked_knee, indices_sitting, indices_lowweightonfeet, knee_flexion_sound);
        [weightratio_transit_vec, boxplotlabel_vec] = UpdateVectors(isubject, indices_transit, trialname, F_GRF_sound, F_GRF_pros, weightratio_transit_vec, boxplotlabel_vec);

        processedData = UpdateProcessedData(ifolder, isubject, trialname, rangeFolders, ...
            knee_flexion_sound, knee_flexion_pros, M_knee_L, M_knee_R, processedData);
    end
end

%% partition and further process data
partitionedData = CalcPartitionedData(processedData);

[phi_knee_S_stand_interp, phi_knee_P_stand_interp, M_knee_S_stand_mean, M_knee_P_stand_mean, ...
    phi_knee_S_sit_interp, phi_knee_P_sit_interp, M_knee_S_sit_mean, M_knee_P_sit_mean] = CalcAverageCurves(partitionedData);

meanWorkDone = CalcWorkDone(partitionedData);

[meanStandTime, meanSitTime, stdStandTime, stdSitTime] = CalcMeanStandSitTimes(partitionedData);
%% plot figures for paper
PlotAverageCurves(phi_knee_S_stand_interp, phi_knee_P_stand_interp, M_knee_S_stand_mean, M_knee_P_stand_mean, ...
    phi_knee_S_sit_interp, phi_knee_P_sit_interp, M_knee_S_sit_mean, M_knee_P_sit_mean);

PlotMeanStandSitTimes(meanStandTime, meanSitTime, stdStandTime, stdSitTime);
